<?php

namespace IPS\richdiscord\Discord;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
    header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
    exit;
}

class _CDNObject
{
    const CDN_URL = "https://cdn.discordapp.com";

    const F_JPEG = '.jpg';
    const F_PNG = '.png';
    const F_WEBP = '.webp';
    const F_GIF = '.gif';

    protected $response;
    protected $url;

    /**
     * CDNObject constructor.
     * @param \IPS\Http\Url $url
     * @param \IPS\Http\Response $response
     */
    protected function __construct( \IPS\Http\Url $url, \IPS\Http\Response $response )
    {
        $this->response = $response;
        $this->url = $url;
    }

    /**
     * Returns response from Discord's CDN.
     * @return \IPS\Http\Response
     */
    public function response()
    {
        return $this->response;
    }

    /**
     * Returns URL to image.
     * @return \IPS\Http\Url
     */
    public function url()
    {
        return $this->url;
    }

    /**
     * Returns an instance of \IPS\Image.
     * @return \IPS\Image
     */
    public function image()
    {
        return \IPS\Image::create( $this->response->content );
    }

    /**
     * Resolves image from CDN.
     * @param  string $path   CDN path
     * @param  string $format File format
     * @return static
     */
    protected static function find( $path, $format )
    {
        $url = \IPS\Http\Url::external( static::CDN_URL . $path, $format );
        $response = $url->request()->get();
        if ( $response->httpResponseCode == "404" )
        {
            throw new \OutOfRangeException();
        }
        return new static( $url, $response );
    }

    /**
     * Resolves Discord emoji.
     * @param  int|string $id     Emoji ID
     * @param  string     $format File format
     * @return static
     */
    public static function emoji( $id, $format )
    {
        if (!in_array($format, [static::F_PNG, static::F_GIF])) throw new \OutOfRangeException();
        return static::find( "/emojis/$id", $format );
    }

    /**
     * Resolves guild's icon.
     * @param  int|string $id     Guild ID
     * @param  string     $hash   Icon hash
     * @param  string     $format File format
     * @return static
     */
    public static function guildIcon( $id, $hash, $format )
    {
        if (!in_array($format, [static::F_PNG, static::F_JPEG, static::F_WEBP])) throw new \OutOfRangeException();
        return static::find( "/icons/$id/$hash", $format );
    }

    /**
     * Resolves guild's splash.
     * @param  int|string $id     Guild ID
     * @param  string     $hash   Splash hash
     * @param  string     $format File format
     * @return static
     */
    public static function guildSplash( $id, $hash, $format )
    {
        if (!in_array($format, [static::F_PNG, static::F_JPEG, static::F_WEBP])) throw new \OutOfRangeException();
        return static::find( "/splashes/$id/$hash", $format );
    }

    /**
     * Resolves user's default avatar
     * @param  int|string $discriminator User's discriminator
     * @return static
     */
    public static function defaultAvatar( $discriminator )
    {
        $discriminator = $discriminator % 5;
        return static::find( "/embed/avatars/$discriminator", static::F_PNG );
    }

    /**
     * Resolves user's avatar.
     * @param  int|string $id     User ID
     * @param  string     $hash   Avatar hash
     * @param  string     $format File format
     * @return static
     */
    public static function avatar( $id, $hash, $format )
    {
        if (!in_array($format, [static::F_PNG, static::F_JPEG, static::F_WEBP, static::F_GIF])) throw new \OutOfRangeException();
        return static::find( "/avatars/$id/$hash", $format );
    }

    /**
     * Resolves application's id.
     * @param  int|string $id     Application ID
     * @param  string     $hash   Icon hash
     * @param  string     $format File format
     * @return static
     */
    public static function appIcon( $id, $hash, $format )
    {
        if (!in_array($format, [static::F_PNG, static::F_JPEG, static::F_WEBP])) throw new \OutOfRangeException();
        return static::find( "/app-icons/$id/$hash", $format );
    }
}